/* File:      gasdev.C
   Date:      12/31/01  (written fall 2001)
   Author:    C. Paciorek
   Purpose:   generate a normally-distributed random variable
   Compile:   g++ -c
   Usage:     
   Arguments: 
   Requires:  base.h, ran1.h
   Reference: Numerical Recipes in C - taken directly from this source
*/

#include "base.h"
#include "ran1.h"

double gasdev(int *idum) {  // generates 2 random deviates at a time; idum is seed
  float ran1(int *idum,int mode,char* line); 
  static int iset=0; 
  static float gset; 
  float fac,rsq,v1,v2; 

  if (*idum < 0) iset=0; 
  if (iset == 0) {  //We don't have an extra deviate handy, so 
    do { v1=2.0*ran1(idum,GENERATE,"a")-1.0; // pick two uniform numbers in the square extending from -1 to +1 in each direction, 
    v2=2.0*ran1(idum,GENERATE,"a")-1.0; 
    rsq=v1*v1+v2*v2; // see if they are in the unit circle,
    } 
    while (rsq >= 1.0 || rsq == 0.0); // and if they are not, try again. 
    fac=sqrt(-2.0*log(rsq)/rsq); /* Now make the Box-Muller transformation to get two normal deviates. Return one and save the other for next time. */
    gset=v1*fac; 
    iset=1; // Set ag. 
    return (double) v2*fac; } 
  else { // We have an extra deviate handy, 
    iset=0; // so unset the ag, 
    return (double) gset; // and return it. 
  } 
}
